NMFdeconvolution <- 
  function(
    cdf.dat = cdf.dat,
    fcnum.nmf = 5,
    fcnum = 4,
    malgo = "Frobenius",
    salgo = "nndsvd",
    ext.watershed =1, #integer
    precision = 0.1,
    MPeriod = 4,
    MSmode = "HRscan",
    MSthreshold = 0,
    lowestMSset = 30,
    highestMSset = 600,
    MDFset =0,
    MDFfilter = 0,
    DB.option = 0
  )
  {
    msrange <- seq(lowestMSset,highestMSset,precision)
    msrange <- round(msrange/precision)*precision
    
    phase.shift = 0
    MSint.digits = 0
    if ( precision >= 1 ){ 
      max.layer.num = 3
    } else {
      max.layer.num = 1
    }
    
    
    if (MSmode == "scan"){
      MDFset <- 0
    }
    
    ###20200604
    old.op <- options(max.print=9999999)
    #flag <- ncvar_get(cdf.dat,"flag_count")
    scantime <- ncvar_get(cdf.dat,"scan_acquisition_time")
    #scannum <- ncvar_get(cdf.dat,"actual_scan_number")
    #medmsmax <- ncvar_get(cdf.dat,"mass_range_max")
    #medmsmin <- ncvar_get(cdf.dat,"mass_range_min")
    #ionid <- ncvar_get(cdf.dat,"scan_index")
    eachscannum <- ncvar_get(cdf.dat,"point_count")
    MStotint <- ncvar_get(cdf.dat,"total_intensity")
    MSvalue <- ncvar_get(cdf.dat,"mass_values")
    MSint <- round(ncvar_get(cdf.dat,"intensity_values"),digit=MSint.digits)
    ###20200604

    #20220224
    flag <- as.array( rep(0,length(scantime)) )
    ionid <- flag
    for(ionid.i in 1:length(eachscannum) ){
      if (ionid.i==1){
        ionid[ionid.i] <- sum(eachscannum[1])
      } else {
        ionid[ionid.i] <- sum(eachscannum[1:(ionid.i-1)])
      } #if
    }
    #20220224
    
    Timepara <- scantime[which(abs(eachscannum)<.Machine$integer.max )]
    RTini <- min( Timepara )/60
    RTruntime <- max( Timepara )/60-RTini
    SamRate <- 1/(mean( Timepara[2:length(Timepara)]-Timepara[1:(length(Timepara)-1) ] ))
    
    pixelnum <- length(flag)
    maxscannum <- max(eachscannum)
    minscannum <- min(eachscannum)
    MSdatabox <- matrix(0,pixelnum, maxscannum+1 )
    MSvaluebox <- MSdatabox
    MSintbox <- MSdatabox
    
    for (inum in 1:pixelnum) {
      if (abs(eachscannum[inum])<.Machine$integer.max){
        initial <- sum(eachscannum[1:inum])-eachscannum[inum]+1
        acqrange <- initial:(initial+eachscannum[inum]-1)
        if(eachscannum[inum]==0){acqrange <-0}
        remainrep <- rep(0,maxscannum-eachscannum[inum]+1 )
        MSvaluebox[inum,] <- c( MSvalue[ acqrange ], remainrep )
        MSintbox[inum,] <- c( MSint[ acqrange ], remainrep )
      }
    }
    
    rownames(MSvaluebox) <-1:(pixelnum)
    rownames(MSintbox) <-1:(pixelnum)
    MSvaluebox[which(MSintbox > 9e+36)] <- -1
    MSintbox[which(MSintbox > 9e+36)] <- -1
    
    
    #Filter MSint Threshold
    MSintbox[which(MSintbox < MSthreshold)] <-0
    MSvaluebox[which(MSintbox < MSthreshold)] <-0
    
    
    #Filter MDF
    if ( MDFfilter==1){
      posnum <- which( (MSvaluebox - floor(MSvaluebox) )  <= 1 + MDFset )
      MSvaluebox[posnum] <- 0
      MSintbox[posnum] <-0
    }
    
    
    #Filter MSrange
    MSintbox[which(MSvaluebox <= lowestMSset)] <-0
    MSvaluebox[which(MSvaluebox <= lowestMSset)] <-0
    MSintbox[which(MSvaluebox >= highestMSset)] <-0
    MSvaluebox[which(MSvaluebox >= highestMSset)] <-0
    
    
    
    #2D TIC data matrix for picture
    originalmatrix <- seq(1, length(flag)) 
    blankbox2original <-  rep(0,length(flag)) 
    usualid <- 0
    
    row <- round(SamRate*MPeriod)
    col <- round(length(scantime)/row)
    
    redidu <- row-SamRate*MPeriod
    redidu.accum <- 1
    blank.id <- round(phase.shift*SamRate+1)
    cont.num <- 0
    tic.box <- as.matrix( rowSums(MSintbox)  )
    mat <- matrix(1,row,col)
    pixel.idbox <- mat
    
    try(silent =TRUE,for (u in round(phase.shift*SamRate+1):(length(tic.box[,1])+round(phase.shift*SamRate)) ){
      usualid <- usualid + 1
      
      #No correction for phase.shift, data lose
      #if ( u%%row==0 ){
      #redidu.accum <-redidu.accum + redidu
      #}
      
      if ( redidu.accum > 1 ){
        blank.id <- blank.id +1
        mat[row*ceiling(blank.id/row)-(blank.id-1),ceiling( (u-round(phase.shift*SamRate))/row )] <- tic.box[round(u-phase.shift*SamRate),]
        if ( blank.id > 0 ){
          blankbox2original[blank.id] <- originalmatrix[usualid]
        }
        blank.id <- blank.id +1
        mat[row*ceiling(blank.id/row)-(blank.id-1),ceiling( (u-round(phase.shift*SamRate))/row )] <- tic.box[round(u-phase.shift*SamRate),]
        if ( blank.id > 0 ){
          blankbox2original[blank.id] <- originalmatrix[usualid]
        }
        redidu.accum <- redidu.accum -1 
      } else {
        blank.id <- blank.id +1
        mat[row*ceiling(blank.id/row)-(blank.id-1),ceiling( (u-round(phase.shift*SamRate))/row )] <- tic.box[round(u-phase.shift*SamRate),]
        if (blank.id > 0){
          blankbox2original[blank.id] <- originalmatrix[usualid]
        }
      }
      
    })
    #2D TIC data matrix for picture
    
    #Graphics
    zz1 <- t(apply(as.matrix(mat[1:nrow(mat),1:ncol(mat)]),2,rev))
    xx1 <- (1:nrow(zz1))/col*RTruntime+RTini
    yy1 <-(((1:ncol(zz1))/row*MPeriod))
    max.zz<-max(zz1)
    
    #20200604 Exit, if invalid data
    if ( all(zz1==0|zz1==Inf|is.na(zz1)) ){
      return("Invalid data. All the data are '0' or 'Inf' or 'NA'.")
    }
    #20200604
    
    
    windows(width=10,height=5)
    par(mar=c(5,7,1,1))
    filled.contour(xx1,yy1,zz1,xlab="GC1 (min)",ylab="GC2 (sec)",col=terrain.colors(30),levels=seq(0,max(tic.box)+0.1,max(tic.box)/30+0.1) )
    
    #Combine three 2DTIC
    mat1 <- c(rep(0,row),mat[1:(length(mat)-row)])
    mat1 <- matrix(mat1,row,col)
    mat2 <- c(mat)
    mat2 <- matrix(mat2,row,col)
    mat3 <- c(mat[(row+1):length(mat)],rep(0,row))
    mat3 <- matrix(mat3,row,col)
    
    com.mat <- 0
    for (roop.col in 1:col){
      com.mat.new <- c(mat1[,roop.col],mat2[,roop.col],mat3[,roop.col])
      com.mat <- c(com.mat,com.mat.new)
    }
    
    com.mat <- com.mat[-c(1)]
    com.mat <- matrix(com.mat,3*row,col)
    com.watershed1 <- watershed(com.mat[1:nrow(com.mat),1:ncol(com.mat)], ext=ext.watershed)
    com.watershed2 <- com.watershed1
    
    for (wat.col.num in 1:(col-1) ){
      if (com.watershed1[2*row,wat.col.num]==com.watershed1[2*row+1,wat.col.num]){
        target.watid <- com.watershed1[1*row+1,wat.col.num+1]
        mod.watid <- com.watershed1[2*row,wat.col.num]
        com.watershed2[which(com.watershed1==target.watid)] <- mod.watid
      }
    }
    
    zz2 <- 1/t(apply( com.watershed2[(row+1):(2*row),],2,rev )) 
    
    #Watershed algorithm
    xx2 <- (1:nrow(zz2))/col*RTruntime+RTini
    yy2 <-(((1:ncol(zz2))/row*MPeriod))
    
    
    windows(width=10,height=5)
    par(mar=c(5,7,1,1))
    
    
    ###20200604
    zz2r <- replace(zz2,which(zz2==Inf),0)
    max.zz <- max(zz2r)
    min.zz <- min(zz2r)
    
    filled.contour(xx2,yy2,zz2r,xlab="GC1 (min)",ylab="GC2 (sec)",col=terrain.colors(30),levels=seq(0,max.zz/100+exp(-100),max.zz/100/30+exp(-100)) )
    ###20200604
    
    
    #Extract peaktop
    zz3.pre <- t(apply(com.watershed2[(row+1):(2*row),],2,rev))
    zz3 <- apply(apply(zz3.pre,1,rev),2,rev)
    zz1 <- t(zz1) 
    
    MSvaluebox.round <- round(MSvaluebox/precision)*precision
    zz3line <- as.numeric(zz3)
    
    if ( length(zz3) > nrow(MSvaluebox.round) ){
      MSvaluebox.round <- rbind(MSvaluebox.round,matrix(0,length(zz3)-nrow(MSvaluebox.round),ncol(MSvaluebox.round)) )
      MSintbox <- rbind(MSintbox,matrix(0,length(zz3)-nrow(MSintbox),ncol(MSintbox)) )
    }
    
    if ( length(zz3) < nrow(MSvaluebox.round) ){
      MSvaluebox.round <- MSvaluebox.round[1:length(zz3),]
      MSintbox <- MSintbox[1:length(zz3),]
    } #if
    
    
    if (fcnum.nmf != 0 & fcnum !=0){
      #devi-loop to supress memory size
      zero.watershed <- 0
      drange0.count <- 0
      for ( devi in 1:ceiling(fcnum/max.layer.num) ) {
        text.list <-rep("matrix(0,length(zz3),length(msrange))",max.layer.num)
        text.list.paste <- paste( text.list,sep="",collapse="," ) 
        list.databox.fact <- eval(parse(text=paste("list(", text.list.paste,  ")",sep="") ))
        
        windows(width=4,height=1)
        par(mar=c(0,0,0,0))
        plot(0:1,0:1,type="n",axes=F,ann=F)
        
        
        for (watershed.id in sort(unique(as.numeric(zz3[zz3>0]))) ){
          segments(0.08,0.6,0.92,0.6,lwd=70, col="white")
          text(0.4,0.8,paste("current watershed: ",watershed.id))
          rect(0,0.1,1,0.1,lwd=2)
          segments(0,0.1,which(sort(unique(as.numeric(zz3[zz3>0])))==watershed.id)/(length(sort(unique(as.numeric(zz3[zz3>0]))))),0.1,col="red",lwd=4)
          
          #Error avoidance. Exception for the case of only 1 row
          if (length(which(zz3==watershed.id)) == 1){
            drange <- 1:length(which(zz3==watershed.id))
            focused.cell.zz3 <- which(zz3==watershed.id)
            MSvaluebox.focused <- t(MSvaluebox.round[which(zz3==watershed.id),])
            msrange.ext <- unique(as.numeric(MSvaluebox.focused))
            msrange.ext <- msrange.ext[msrange.ext>0]
            databox <- matrix(0,length(drange),length(msrange.ext) )
            colnames(databox) <- msrange.ext
            MSintbox.focused <- t(MSintbox[focused.cell.zz3,])
            drange0.count <- drange0.count + 1
          }
          
          if (length(which(zz3==watershed.id)) > 1){
            drange <- 1:length(which(zz3==watershed.id))
            MSvaluebox.focused <- MSvaluebox.round[which(zz3==watershed.id),]
            focused.cell.zz3 <- which(zz3==watershed.id)
            MSintbox.focused <- MSintbox[focused.cell.zz3,]
            msrange.ext <- unique(as.numeric(MSvaluebox.focused[drange,]))
            msrange.ext <- msrange.ext[msrange.ext>0]
            databox <- matrix(0,length(drange),length(msrange.ext) )
            colnames(databox) <- msrange.ext
          }
          
          #Data matrix for nmf. Duplicated m/z is summed.
          count.i <- 0
          for (i in drange){
            count.i <- count.i + 1
            matms.databox <- match(msrange.ext,MSvaluebox.focused[count.i,],nomatch=0)
            matms.databox <- matms.databox[matms.databox>0]
            matms.databox.inv <- match(MSvaluebox.focused[count.i,],msrange.ext,nomatch=0)
            matms.databox.inv <- matms.databox.inv[matms.databox.inv>0]
            
            count.iii <- 0
            box.iii <- rep(0,length(matms.databox) )
            for (iii in unique(MSvaluebox.focused[count.i,][MSvaluebox.focused[count.i,]>0]) ){
              count.iii <- count.iii +1
              box.iii[count.iii] <- sum(MSintbox.focused[count.i,which(MSvaluebox.focused[count.i,]==iii)])
            }
            databox[count.i,unique(matms.databox.inv)] <- box.iii
          }
          
          #Error avoidance. Remove columns, which don't include m/z value, for nmf calculation execution.
          if ( sum(databox)>0 ){
            delete.col <- which( colSums( databox>0  ) == 0)
          } else {
            delete.col <- 0
            databox <- matrix(0,1,1)
          }
          
          if (sum(delete.col)>0){
            databox <- databox[,-c(delete.col)]
          }
          
          #Error avoidance. In case the rows don't have any m/z value, 1 is added in their intensity
          if( any(rowSums(databox)==0) ){
            databox[,1] <- databox[,1] +1
          }
          
          #Error avoidance. ica dosen't work for only 1 datum
          remove.colnum <- which( colSums(databox > 0)<=1 )
          if ( ncol(databox) > 1 ){
            databox <- databox[,-c(remove.colnum)]
            if ( class(databox)[1]=="numeric" ){ #20220218
              databox <- as.matrix(databox)
            }
            
          }
          
          if ( sum(databox)!=0 ){
            remove.rownum <- which( rowSums(databox)==0 )
            if ( nrow(databox) > 1 & sum(remove.rownum)>0 ){
              databox <- databox[-c(remove.rownum),]
              focused.cell.zz3 <- focused.cell.zz3[-c(remove.rownum)]
            }
            if ( class(databox)[1]=="numeric" ){ #20220218
              databox <- t(as.matrix(databox))
            }
          } else {
            databox <- as.matrix(1,1,1)
          }
          
          
          ###20170203###
          databox <- t(databox) #20170202
          #Error avoidance. nmf dosen't work for only 1 case (1 row)
          if (nrow(databox) > 1 & ncol(databox) > 1 ){	
            if (nrow(databox)>=fcnum.nmf & ncol(databox)>=fcnum.nmf ){
              resnmf <- nmf(databox, fcnum.nmf,seed=salgo, malgo )
            } else if ( nrow(databox) < fcnum.nmf | ncol(databox) < fcnum.nmf ) {
              num.factor <- min( nrow(databox), ncol(databox) )
              resnmf <- nmf(databox, num.factor,seed=salgo, malgo )
            }
            
            resnmf.coef <- coef(resnmf)
            resnmf.basis <- basis(resnmf)
            
            #Peak top with high intensity in TIC is placed in upper layer
            factor.coef <- apply( resnmf.coef,1,max)  #factor.coef <- apply( resnmf.basis,2,max) 20170202
            resnum.basis.factored <- t( factor.coef * t(resnmf.basis) ) #resnum.coef.factored <- factor.coef * resnmf.coef 20170203
            ch.order <- order( colSums(resnum.basis.factored),decreasing=T ) #ch.order <- order( rowSums(resnum.coef.factored),decreasing=T ) 20170202
            resnmf.coef <- resnmf.coef[ch.order,]
            resnmf.basis <- resnmf.basis[,ch.order]
            contrib <- rowSums(resnmf.coef)
            
            #Calculate up to the factor with 0.01*factor1, otherwise factors are set not to be found.
            usefc.num.max <- length(which(contrib/contrib[1] > 0.01))
            usefc.num <- (max.layer.num*devi-max.layer.num+1):(max.layer.num*devi)  
            
            if ( (max.layer.num*devi-max.layer.num+1) <= usefc.num.max ){		
              if ( max(usefc.num) > usefc.num.max ){
                usefc.num <- (max.layer.num*devi-max.layer.num+1):usefc.num.max
              }
            }
            
            if( min(usefc.num) > usefc.num.max ){
              usefc.num <- usefc.num.max
            }
            
            #Store list.databox.fact for each layer.
            if ( usefc.num.max >= (max.layer.num*devi-max.layer.num+1) ){
              layer.count <- 0
              for ( fc.rep in usefc.num ){
                layer.count <- layer.count + 1 
                nmf.load <- resnmf.basis[,fc.rep] #nmf.load <- resnmf.coef[fc.rep,] 20170202
                nmf.score <- resnmf.coef[fc.rep,]  #nmf.score <- resnmf.coef[fc.rep,] 20170202
                nmfload.eachrow <- nmf.score %*% t(nmf.load)
                nmfload.ms <- as.numeric(colnames(nmfload.eachrow))
                databox.input <- matrix(0,nrow(nmfload.eachrow),length(msrange))
                colnames(databox.input) <- msrange
                mid <- match(round(nmfload.ms/precision)*precision,msrange)
                databox.input[,mid] <- nmfload.eachrow
                list.databox.fact[[ layer.count ]][focused.cell.zz3,] <- databox.input
              }
              ###20170203###
              
              
              #Just look for watershed with TIC 0.
              list.tex.pre <- paste( "list.databox.fact[[",1:max.layer.num,"]][focused.cell.zz3,]",collapse="," )
              wat.sum.nmfload <- eval(parse(text=paste("sum(",list.tex.pre,")") ))
              if (wat.sum.nmfload==0){
                zero.watershed <- rbind(zero.watershed,watershed.id)
              }
              
              #Error avoidance. nmf dosen't work for only 1 case (1 row)
            } else {
              list.databox.fact[[ 1 ]][focused.cell.zz3,as.numeric(colnames(databox))-lowestMSset+1] <- as.matrix(databox/colSums(databox) ) 
              
              #Just look for watershed with TIC 0.
              list.tex.pre <- paste( "list.databox.fact[[",1:max.layer.num,"]][focused.cell.zz3,]",collapse="," )
              wat.sum.nmfload <- eval(parse(text=paste("sum(",list.tex.pre,")") ))
              if (wat.sum.nmfload==0){
                zero.watershed <- cbind(zero.watershed,watershed.id)
              }
              
            } # end if
          }
        }
        gc()
        gc()
        
        
        #Convert to rounded m/z to original m/z values.
        layer.count <- 0
        for ( fc.rep in (max.layer.num*devi-max.layer.num+1):(max.layer.num*devi)   ){
          layer.count <- layer.count +1
          databox.relative <- list.databox.fact[[ layer.count ]]
          MSintbox.deconv <- matrix(0,length(MSintbox[,1]),length(MSintbox[1,]))
          
          for (inprow.num in 1:length(MSintbox[,1]) ) {
            ms.locate <- match(MSvaluebox.round[inprow.num,],msrange,nomatch=0)
            ms.locate <- unique(ms.locate[ms.locate>0])
            ms.locate.inv <- match(msrange,MSvaluebox.round[inprow.num,],nomatch=0)
            ms.locate.inv <- ms.locate.inv[ms.locate.inv>0]
            MSintbox.deconv[inprow.num, ms.locate.inv] <- databox.relative[inprow.num,ms.locate]
          }
          eval(parse(text=paste("MSintbox.deconv", fc.rep, " <- MSintbox.deconv",sep="")))
        }
        
        
        #For memory suppression, list.databox.fact is deleted.
        rm(list.databox.fact,databox.relative)
        gc()
        gc()
        
        #TIC dipiction.
        for ( fc.rep in (max.layer.num*devi-max.layer.num+1):(max.layer.num*devi)   ){
          windows(width=10,height=5)
          test.pic <- eval(parse(text=paste("matrix(rowSums(MSintbox.deconv",fc.rep,"),nrow(mat),ncol(mat))",sep="") ))
          zz4 <- t(as.matrix(test.pic[1:nrow(mat),1:ncol(mat)]))
          filled.contour(xx1,yy1,zz4,xlab="GC1 (min)",ylab="GC2 (sec)",col=terrain.colors(30),levels=seq(0,max(zz4/10)+0.1,max(zz4/10)/30+0.1) )
        }
        
        gc()
        gc()
        
        
        #CDF output
        for ( fc.rep in (max.layer.num*devi-max.layer.num+1):(max.layer.num*devi)   ){
          if ( nrow(MSintbox) > length(eachscannum) ){
            loop.length <- length(eachscannum)
          }
          if ( nrow(MSintbox) <= length(eachscannum) ){
            loop.length <- nrow(MSintbox)
          }
          
          for (i in 1:loop.length ) {
            if (abs(eachscannum[i])<.Machine$integer.max){
              initial <- sum(eachscannum[1:i])-eachscannum[i]+1
              acqrange <- initial:(initial+eachscannum[i]-1)
              if(eachscannum[i]==0){acqrange <-0}
              MSvalue[acqrange] <- MSvaluebox[i, 1:length(acqrange) ]
              MSint[acqrange] <- eval(parse(text=paste("MSintbox.deconv",fc.rep,"[i, 1:length(acqrange) ]",sep="")))
            }
          }
          
          ###20200604
          MStotint <- eval(parse(text=paste("rowSums(MSintbox.deconv",fc.rep,")",sep="")))
          
          
          scan_number <- ncdim_def( name="scan_number", units='',vals=1:length(scantime) ) 
          point_number <- ncdim_def( name="point_number", units='',vals=1:length(MSint) ) 
          
          vardimfirst <- ncvar_def( name="flag_count",units="",dim=scan_number,missval=1, longname="flag_count",prec="integer" )
          vardimA <- ncvar_def( name="scan_acquisition_time", units="",dim=scan_number,missval=-1,longname="scan_acquisition_time",prec="double" )
          #vardimB <- ncvar_def( name="actual_scan_number", units="",dim=scan_number,missval=-1,longname="actual_scan_number",prec="integer" )
          #vardimC <- ncvar_def( name="mass_range_max", units="",dim=scan_number,missval=-1,longname="mass_range_max",prec="double" )
          #vardimD <- ncvar_def( name="mass_range_min", units="",dim=scan_number,missval=-1,longname="mass_range_min",prec="double" )
          vardimE <- ncvar_def( name="scan_index", units="",dim=scan_number,missval=-1,longname="scan_index",prec="integer" )
          vardimF <- ncvar_def( name="point_count", units="",dim=scan_number,missval=-1,longname="point_count",prec="integer" )
          vardimG <- ncvar_def( name="total_intensity", units="",dim=scan_number,missval=-1,longname="total_intensity",prec="double" )
          vardimH <- ncvar_def( name="mass_values", units="",dim=point_number,missval=-1,longname="mass_values",prec="double" )
          vardimI <- ncvar_def( name="intensity_values", units="",dim=point_number,missval=-1,longname="intensity_values",prec="single" )
          
          ncnew <- nc_create( paste(savefilename,"_layer",fc.rep,".cdf",sep=""), list(vardimfirst,vardimA,vardimE,vardimF,vardimG,vardimH,vardimI) )
          
          ncvar_put( nc=ncnew,varid=vardimfirst,vals=flag )
          ncvar_put( nc=ncnew,varid=vardimA,vals=scantime )
          #ncvar_put( nc=ncnew,varid=vardimB,vals=scannum )
          ncvar_put( nc=ncnew,varid=vardimE,vals=ionid )
          ncvar_put( nc=ncnew,varid=vardimF,vals=eachscannum )
          ncvar_put( nc=ncnew,varid=vardimG,vals=MStotint )
          ncvar_put( nc=ncnew,varid=vardimH,vals=MSvalue )
          ncvar_put( nc=ncnew,varid=vardimI,vals=MSint )
          
          nc_close(ncnew)
          ###20200604
          
          eval(parse(text=paste("rm( MSintbox.deconv",fc.rep,")",sep="")))
          gc()
          gc()
          
        }
      }
    }
    
    if(DB.option==1){
      MSpeaklist(savefilename, pos,fcnum,MPeriod,MSthreshold,lowestMSset, highestMSset,MDFset,MDFfilter)
      cat("2MSpeaklist.r was also performed.")
    }
  }
